@@ -2,7 +2,7 @@ package PICA::Path;
 use v5.14.1;
 use utf8;
 
-our $VERSION = '1.25';
+our $VERSION = '1.26';
 
 use Carp qw(confess);
 use Scalar::Util qw(reftype);
@@ -183,7 +183,7 @@ sub match_subfields {
         }
 
         my $subfields = $self->[2];
-        $subfields =~ s{.*\[(.+)\].*}{\1}g;
+        $subfields =~ s{.*\[(.+)\].*}{$1}g;
         for my $subfield (split('', $subfields)) {
             my $value = $subfield_href->{$subfield} // [undef];
             if (defined $from) {
